@@ -42,24 +42,25 @@ This file is available from http://sourceforge.net/projects/latex2rtf/
 #include "counters.h"
 #include "preamble.h"
 #include "vertical.h"
-
-static void TranslateCommand(); /* converts commands */
+#include "fields.h"
 
 static int ret = 0;
 
+static void TranslateCommand(void);
+
 void ConvertString(const char *string)
 
 /******************************************************************************
      purpose : converts string in TeX-format to Rtf-format
  ******************************************************************************/
 {
-    if (string == NULL)
+    if (string == NULL || string == '\0')
         return;
 
     if (PushSource(NULL, string) == 0) {
         diagnostics(5, "Entering Convert() from ConvertString()");
 
-		show_string(5, string, "convert");
+        show_string(4, string, "converting");
 
         while (StillSource())
             Convert();
@@ -73,16 +74,16 @@ void ConvertAllttString(char *s)
 
 /******************************************************************************
      purpose : converts string in TeX-format to Rtf-format
-			   according to the alltt environment, which is like
-			   verbatim environment except that \, {, and } have
-			   their usual meanings
+               according to the alltt environment, which is like
+               verbatim environment except that \, {, and } have
+               their usual meanings
 ******************************************************************************/
 {
     char cThis;
 
     if (s == NULL)
         return;
-    diagnostics(4, "Entering Convert() from StringAllttConvert()");
+    diagnostics(3, "Entering Convert() from StringAllttConvert()");
 
     if (PushSource(NULL, s) == 0) {
 
@@ -114,10 +115,10 @@ void ConvertAllttString(char *s)
         }
         PopSource();
     }
-    diagnostics(4, "Exiting Convert() from StringAllttConvert()");
+    diagnostics(3, "Exiting Convert() from StringAllttConvert()");
 }
 
-void Convert()
+void Convert(void)
 
 /****************************************************************************
 purpose: converts inputfile and writes result to outputfile
@@ -126,9 +127,9 @@ purpose: converts inputfile and writes result to outputfile
     char cThis = '\n';
     char cLast = '\0';
     char cNext;
-    int count, pending_new_paragraph;
+    int count;
 
-    diagnostics(5, "Entering Convert ret = %d", ret);
+    diagnostics(3, "Entering Convert ret = %d", ret);
     RecursionLevel++;
     PushLevels();
 
@@ -141,56 +142,49 @@ purpose: converts inputfile and writes result to outputfile
             diagnostics(6, "Current character is '%c' mode = %d ret = %d level = %d", cThis, getTexMode(), ret,
               RecursionLevel);
 
-        pending_new_paragraph--;
+        /* preliminary support for utf8 sequences.  Thanks to CSH */
+        if ((cThis & 0x8000) && (CurrentFontEncoding() == ENCODING_UTF8)) {
+            uint8_t byte;
+            uint16_t len, value, i;
 
-		/* preliminary support for utf8 sequences.  Thanks to CSH */
-        if ((cThis & 0x8000) && (strcmp(g_charset_encoding_name, "utf8") == 0)) {
-        	unsigned char byte;
-        	unsigned int len, value, i;
-
-        	/* Get the number of bytes in the sequence        */
-        	/* Must use an unsigned character for comparisons */
-        	byte = cThis;
-        	if (byte >= 0xF0) { 
-        		len = 3; 
-        		value = byte & ~0xF0; 
-        	} 
-        	else if (byte >= 0xE0) { 
-        		len = 2; 
-        		value = byte & ~0xE0; 
-        	}		
-        	else if (byte >= 0xC0) { 
-        		len = 1; 
-        		value = byte & ~0xC0; 
-        	}
-        	
-        	/* reassemble the character */
-        	for ( i=0; i<len; i++) {
-        		byte = getTexChar() & ~0xC0;
-        		value = (value << 6) + byte;
-        	}
-        	
-        	diagnostics(4,"(flag = 0x%X) char value = 0X%04X or %ud (%d bytes)", (unsigned char) cThis, value, value, len);
-
-        	/* values above 0x8000 must be negative! */
-			if (value < 0x8000)
-				fprintRTF("\\u%d?", value);
-			else
-				fprintRTF("\\u%d?", (int)((double)value-0x10000));
-        }        	        	
-		else
+            /* Get the number of bytes in the sequence        */
+            /* Must use an unsigned character for comparisons */
+            byte = cThis;
+            len = 0;
+            value = 0;
+            if (byte >= 0xF0) { 
+                len = 3; 
+                value = byte & ~0xF0; 
+            } 
+            else if (byte >= 0xE0) { 
+                len = 2; 
+                value = byte & ~0xE0; 
+            }       
+            else if (byte >= 0xC0) { 
+                len = 1; 
+                value = byte & ~0xC0; 
+            }
+            
+            /* reassemble the character */
+            for ( i=0; i<len; i++) {
+                byte = getTexChar() & ~0xC0;
+                value = (value << 6) + byte;
+            }
+            
+            diagnostics(4,"(flag = 0x%X) char value = 0X%04X or %u (%u bytes)", (unsigned char) cThis, value, value, len);
+            CmdUnicodeChar(value);
+        }                       
+        else
 
         switch (cThis) {
 
             case '\\':
                 PushLevels();
-
                 TranslateCommand();
-
                 CleanStack();
 
-                if (ret > 0) {
-                    diagnostics(5, "Exiting Convert via TranslateCommand ret = %d level = %d", ret, RecursionLevel);
+                if (ret > 1) {
+                    diagnostics(3, "Exiting Convert via TranslateCommand ret = %d level = %d", ret, RecursionLevel);
                     ret--;
                     RecursionLevel--;
                     return;
@@ -211,8 +205,8 @@ purpose: converts inputfile and writes result to outputfile
                 CleanStack();
                 ret = RecursionLevel - PopBrace();
                 fprintRTF("}");
-                if (ret > 0) {
-                    diagnostics(5, "Exiting Convert via '}' ret = %d level = %d", ret, RecursionLevel);
+                if (ret > 1) {
+                    diagnostics(3, "Exiting Convert via '}' ret = %d level = %d", ret, RecursionLevel);
                     ret--;
                     RecursionLevel--;
                     return;
@@ -234,7 +228,7 @@ purpose: converts inputfile and writes result to outputfile
                 break;
 
             case '\n':
-                tabcounter = 0;
+                g_tab_counter = 0;
 
                 if (getTexMode() == MODE_MATH || getTexMode() == MODE_DISPLAYMATH) {
 
@@ -245,7 +239,6 @@ purpose: converts inputfile and writes result to outputfile
                     cNext = getNonSpace();
 
                     if (cNext == '\n') {    /* new paragraph ... skip all ' ' and '\n' */
-                        pending_new_paragraph = 2;
                         CmdEndParagraph(0);
 
                     } else {    /* add a space if needed */
@@ -269,29 +262,26 @@ purpose: converts inputfile and writes result to outputfile
                 }
                 break;
 
-            case '&':
+            case '&':         
+                /* should occur in array, align, or eqnarray environments  */
+
+                /* are we are in the middle of \begin{array} ... \end{array} */
                 if (g_processing_arrays) {
                     fprintRTF("%c", g_field_separator);
                     break;
                 }
-
-                if (getTexMode() == MODE_MATH || getTexMode() == MODE_DISPLAYMATH) {    /* in eqnarray */
-                    if (g_processing_fields) fprintRTF("}}}{\\fldrslt }}");
-					fprintRTF("\\tab ");
-                    if (g_processing_fields) fprintRTF("{{\\field{\\*\\fldinst{ EQ ");
-                    g_equation_column++;
-                    break;
+                
+                /* must be in 'eqnarray' or 'align' environment */
+                if (EQ_field_active()) {
+                    diagnostics(4,"ending/restarting field before alignment '&'");
+                    endCurrentField();
+                    fprintRTF("\\tab\n");
+                    startField(FIELD_EQ);
+                } else {
+                    diagnostics(4,"tabbing to match '&'");
+                    fprintRTF("\\tab\n");
                 }
-
-                if (g_processing_tabular) { /* in tabular */
-                	diagnostics(0,"this should not happen tabular should handle this!");
-                	/*
-                    actCol++;
-                    fprintRTF("\\cell\\pard\\intbl\\q%c ", colFmt[actCol]);
-                    */
-                    break;
-                }
-                fprintRTF("&");
+                g_equation_column++;               
                 break;
 
             case '~':
@@ -308,7 +298,7 @@ purpose: converts inputfile and writes result to outputfile
 
             case '-':
                 if (getTexMode() == MODE_MATH || getTexMode() == MODE_DISPLAYMATH)
-                	CmdSymbolChar(0x2d);
+                    CmdUnicodeChar(8722); /* MINUS SIGN */
                 else {
                     changeTexMode(MODE_HORIZONTAL);
 
@@ -372,17 +362,7 @@ purpose: converts inputfile and writes result to outputfile
                 if (getTexMode() == MODE_HORIZONTAL) {
                     cNext = getTexChar();
                     if (cNext == '<') {
-                        if (FrenchMode) {   /* not quite right */
-                            skipSpaces();
-                            cNext = getTexChar();
-                            if (cNext == '~')
-                                skipSpaces();
-                            else
-                                ungetTexChar(cNext);
-                            fprintRTF("\\'ab\\~");
-
-                        } else
-                            fprintRTF("\\'ab");
+                        fprintRTF("\\'ab");
                     } else {
                         ungetTexChar(cNext);
                         fprintRTF("<");
@@ -474,21 +454,21 @@ purpose: converts inputfile and writes result to outputfile
                 break;
 
             case '(':
-                if (g_processing_fields && g_escape_parens)
+                if (processing_fields() && g_escape_parens)
                     fprintRTF("\\\\(");
                 else
                     fprintRTF("(");
                 break;
 
             case ')':
-                if (g_processing_fields && g_escape_parens)
+                if (processing_fields() && g_escape_parens)
                     fprintRTF("\\\\)");
                 else
                     fprintRTF(")");
                 break;
 
             case ';':
-                if (g_field_separator == ';' && g_processing_fields)
+                if (g_field_separator == ';' && processing_fields())
                     fprintRTF("\\\\;");
                 else if (FrenchMode)
                     fprintRTF("\\~;");
@@ -497,14 +477,15 @@ purpose: converts inputfile and writes result to outputfile
                 break;
 
             case ',':
-                if (g_field_separator == ',' && g_processing_fields) {
-                	if (g_processing_arrays) {
-                	/* this is crazy, fields fail if \, is present in EQ array! */
-                	/* substitute semi-colon because it is least worst option   */
-						fprintRTF(";"); 
-                	} else {
-						fprintRTF("\\\\,"); 
-                	}
+            
+                if (g_field_separator == ',' && processing_fields()) {
+                    if (g_processing_arrays) {
+                    /* this is crazy, fields fail if \, is present in EQ array! */
+                    /* substitute semi-colon because it is least worst option   */
+                        fprintRTF(";"); 
+                    } else {
+                        fprintRTF("\\\\,"); 
+                    }
                 } else {
                 /* ,, is \quotedblbase nearly always */
                     if ((cNext = getTexChar()) && cNext == ',') {
@@ -518,48 +499,52 @@ purpose: converts inputfile and writes result to outputfile
 
             default:
                 if (getTexMode() == MODE_MATH || getTexMode() == MODE_DISPLAYMATH) {
-                	if (('a' <= cThis && cThis <= 'z') || ('A' <= cThis && cThis <= 'Z')) {
-                    	if (CurrentFontSeries() == F_SERIES_BOLD)    /* do not italicize */
-                        	fprintRTF("%c", cThis);
+                    if (('a' <= cThis && cThis <= 'z') || ('A' <= cThis && cThis <= 'Z')) {
+                        if (CurrentFontSeries() == F_SERIES_BOLD)    /* do not italicize */
+                            fprintRTF("%c", cThis);
                         else if (CurrentFontShape() == F_SHAPE_MATH_UPRIGHT)
-                        	fprintRTF("%c", cThis);
-						else
-                        	fprintRTF("{\\i %c}", cThis);
-                	} else
+                            fprintRTF("%c", cThis);
+                        else
+                            fprintRTF("{\\i %c}", cThis);
+                    } else
                         fprintRTF("%c", cThis);
                 
 
                 } else {
-
-                    changeTexMode(MODE_HORIZONTAL);
-                    fprintRTF("%c", cThis);
+                    if (getTexMode()==MODE_VERTICAL) 
+                        changeTexMode(MODE_HORIZONTAL);
+                        
+                    if (CurrentFontEncoding() != ENCODING_OT2)
+                        fprintRTF("%c", cThis);
+                    else
+                        CmdOT2Transliteration(cThis);
                 }
                 break;
         }
 
-        tabcounter++;
+        g_tab_counter++;
         cLast = cThis;
     }
     RecursionLevel--;
-    diagnostics(5, "Exiting Convert via exhaustion ret = %d", ret);
+    diagnostics(3, "Exiting Convert via exhaustion ret = %d", ret);
 }
 
-void TranslateCommand()
+static void TranslateCommand(void)
 
 /****************************************************************************
 purpose: The function is called on a backslash in input file and
-	 tries to call the command-function for the following command.
+     tries to call the command-function for the following command.
 returns: success or not
  ****************************************************************************/
 {
     char cCommand[MAXCOMMANDLEN];
-    int i, mode;
+    int i, mode, height;
     int cThis,cNext;
 
     cThis = getTexChar();
     mode = getTexMode();
 
-    diagnostics(5, "Beginning TranslateCommand() \\%c", cThis);
+    diagnostics(4, "Beginning TranslateCommand() \\%c", cThis);
 
     switch (cThis) {
         case 'a':
@@ -607,14 +592,25 @@ returns: success or not
             return;
 
         case '\\':             /* \\[1mm] or \\*[1mm] possible */
-            CmdSlashSlash(0);
+            diagnostics(5,"here we are in TranslateCommand with '\\\\'");
+            height = getSlashSlashParam();
+            if (g_processing_arrays)            /* \begin{array} ... \end{array} */
+                CmdArraySlashSlash(height);     /* may be contained in eqnarray environment */
+            else if (g_processing_eqnarray)
+                CmdEqnArraySlashSlash(height);
+            else  if (g_processing_tabbing)
+                diagnostics(1,"should not be processing \\\\ in tabbing here");
+            else if (g_processing_tabular)
+                diagnostics(1,"should not be processing \\\\ in tabular here");
+            else
+                CmdSlashSlash(height);
             return;
 
         case 0x0D:  /*handle any CRLF that might have snuck in*/
-			cNext = getTexChar();
-			if (cNext != 0x0A)
-				ungetTexChar(cNext);
-			/* fall through */
+            cNext = getTexChar();
+            if (cNext != 0x0A)
+                ungetTexChar(cNext);
+            /* fall through */
 
         case '\t':  /* tabs should already be spaces */
         case ' ':
@@ -634,7 +630,7 @@ returns: success or not
                 (void) PopBrace();
                 PushBrace();
             } else
-				fprintRTF("\\-");
+                fprintRTF("\\-");
             return;
 
         case '+':
@@ -663,7 +659,7 @@ returns: success or not
                 CmdTabjump();
                 PushBrace();
             } else
-                CmdSpace(0.50); /* medium space */
+                CmdSpace((float)0.50); /* medium space */
             return;
 
         case '`':
@@ -710,12 +706,12 @@ returns: success or not
             
             cThis = getTexChar();
             if (cThis=='^') {
-            	/* usually \^^M ... just replace with a blank */
-            	cThis = getTexChar();
-            	fprintRTF(" ");
+                /* usually \^^M ... just replace with a blank */
+                getTexChar();
+                fprintRTF(" ");
             } else {
-            	ungetTexChar(cThis);
-				CmdHatChar(0);
+                ungetTexChar(cThis);
+                CmdHatChar(0);
             }
             return;
         case '.':
@@ -758,7 +754,7 @@ returns: success or not
         case ';':
             if (mode == MODE_VERTICAL)
                 changeTexMode(MODE_HORIZONTAL);
-            CmdSpace(0.75);     /* \; produces a thick space */
+            CmdSpace((float)0.75);     /* \; produces a thick space */
             return;
         case '@':
             CmdIgnore(0);       /* \@ produces an "end of sentence" space */
@@ -774,7 +770,7 @@ returns: success or not
     /* Commands consist of letters and can have an optional * at the end */
     for (i = 0; i < MAXCOMMANDLEN-1; i++) {
         if (!isalpha((int) cThis) && (cThis != '*')) {
-            bool found_nl = FALSE;
+            int found_nl = FALSE;
 
             if (cThis == '%') { /* put the % back and get the next char */
                 ungetTexChar('%');
@@ -782,7 +778,7 @@ returns: success or not
             }
 
             /* all spaces after commands are ignored, a single \n may occur */
-            while (cThis == ' ' || (cThis == '\n' && !found_nl)) {
+            while (cThis == ' ' || (0 && cThis == '\n' && !found_nl)) {
                 if (cThis == '\n')
                     found_nl = TRUE;
                 cThis = getTexChar();
@@ -797,12 +793,12 @@ returns: success or not
     }
 
     cCommand[i] = '\0';         /* mark end of string with zero */
-    diagnostics(5, "TranslateCommand() <%s>", cCommand);
+    diagnostics(4, "TranslateCommand() <%s>", cCommand);
 
-	if (i==MAXCOMMANDLEN-1) {
-	    diagnostics(WARNING, "Skipping absurdly long command <%s>", cCommand);
-	    return;
-	}
+    if (i==MAXCOMMANDLEN-1) {
+        diagnostics(WARNING, "Skipping absurdly long command <%s>", cCommand);
+        return;
+    }
 
     if (i == 0)
         return;
@@ -814,9 +810,9 @@ returns: success or not
 
     if (CallCommandFunc(cCommand)) {    /* call handling function for command */
         if (strcmp(cCommand, "end") == 0) {
-    		diagnostics(5, "before PopBrace()");
+            diagnostics(4, "before PopBrace()");
             ret = RecursionLevel - PopBrace();
-    		diagnostics(5, "after PopBrace(), ret=%d",ret);
+            diagnostics(4, "after PopBrace(), ret=%d",ret);
             fprintRTF("}");
         }
         return;
@@ -828,5 +824,8 @@ returns: success or not
     if (TryVariableIgnore(cCommand))
         return;
 
+    if (TryConditionSet(cCommand))
+        return;
+
     diagnostics(WARNING, "Unknown command '\\%s'", cCommand);
 }
