@@ -1,5 +1,8 @@
+#ifndef _INCLUDED_GRAPHICS_H
+#define _INCLUDED_GRAPHICS_H 1
+
 #define FIGURE_INCLUDEGRAPHICS 1
-#define FIGURE_EPSFFILE         2
+#define FIGURE_EPSFFILE        2
 #define FIGURE_EPSFBOX         3
 #define FIGURE_BOXEDEPSF       4
 #define FIGURE_PSFIG           5
@@ -7,7 +10,7 @@
 void PrepareDisplayedBitmap(char *the_type);
 void FinishDisplayedBitmap(void);
 void WriteLatexAsBitmap(char *pre, char *eq, char *post);
-void PutLatexFile(char *s, double height0, double width0, double scale, char *pre);
+void PutLatexFile(const char *tex_file_stem, double scale, const char *pre);
 void CmdGraphics(int code);
 void CmdPicture(int code);
 void CmdPsPicture(int code);
@@ -15,3 +18,12 @@ void CmdPsGraph(int code);
 void CmdMusic(int code);
 void CmdPsset(int code);
 void CmdNewPsStyle(int code);
+void CmdGraphicsPath(int code);
+
+typedef void PutFileFnc(char *, double, double, double, double);
+
+typedef struct {
+    char       *extension;
+    PutFileFnc *encoder;
+} GraphConvertElement;
+#endif
