@@ -1,7 +1,6 @@
-
 /* main.c - LaTeX to RTF conversion program
 
-Copyright (C) 1995-2007 The Free Software Foundation
+Copyright (C) 1995-2012 The Free Software Foundation
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -20,11 +19,16 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 This file is available from http://sourceforge.net/projects/latex2rtf/
  
 Authors:
-	1995	  Fernando Dorner, Andreas Granzer, Freidrich Polzer, Gerhard Trisko
-	1995-1997 Ralf Schlatterbeck
-	1998-2000 Georg Lehner
-	2001-2007 Scott Prahl
+    1995      Fernando Dorner, Andreas Granzer, Freidrich Polzer, Gerhard Trisko
+    1995-1997 Ralf Schlatterbeck
+    1998-2000 Georg Lehner
+    2001-2007 Scott Prahl
 */
+#if defined(NOSTDERR)
+#define ERROUT stdout
+#else
+#define ERROUT stderr
+#endif 
 
 #include <stdio.h>
 #include <ctype.h>
@@ -53,6 +57,7 @@ Authors:
 #include "xrefs.h"
 #include "preparse.h"
 #include "vertical.h"
+#include "fields.h"
 
 FILE *fRtf = NULL;              /* file pointer to RTF file */
 char *g_tex_name = NULL;
@@ -67,75 +72,74 @@ char *g_bbl_name = NULL;
 char *g_home_dir = NULL;
 
 char *progname;                 /* name of the executable file */
-bool GermanMode = FALSE;        /* support germanstyle */
-bool FrenchMode = FALSE;        /* support frenchstyle */
-bool RussianMode = FALSE;       /* support russianstyle */
-bool CzechMode = FALSE;         /* support czech */
+int SpanishMode = FALSE;       /* support spanishstyle */
+int GermanMode = FALSE;        /* support germanstyle */
+int FrenchMode = FALSE;        /* support frenchstyle */
+int RussianMode = FALSE;       /* support russianstyle */
+int CzechMode = FALSE;         /* support czech */
 
-char g_charset_encoding_name[20] = "cp1252";
-int g_fcharset_number = 0;
-
-bool twoside = FALSE;
+int twoside = FALSE;
 int g_verbosity_level = WARNING;
-bool g_little_endian = FALSE;   /* set properly in main() */
-int g_dots_per_inch = 300;
+int g_little_endian = FALSE;   /* set properly in main() */
+uint16_t g_dots_per_inch = 300;
 
-bool pagenumbering = TRUE;      /* by default use plain style */
+int pagenumbering = TRUE;      /* by default use plain style */
 int headings = FALSE;
 
-bool g_processing_preamble = TRUE;  /* flag set until \begin{document} */
-bool g_processing_figure = FALSE;   /* flag, set for figures and not tables */
-bool g_processing_eqnarray = FALSE; /* flag set when in an eqnarry */
+int g_processing_preamble = TRUE;  /* flag set until \begin{document} */
+int g_processing_figure = FALSE;   /* flag, set for figures and not tables */
+int g_processing_eqnarray = FALSE; /* flag set when in an eqnarry */
 int g_processing_arrays = 0;
-int g_processing_fields = 0;
 
-bool g_show_equation_number = FALSE;
+int g_show_equation_number = FALSE;
 int g_enumerate_depth = 0;
-bool g_suppress_equation_number = FALSE;
-bool g_aux_file_missing = FALSE;    /* assume that it exists */
-bool g_bbl_file_missing = FALSE;    /* assume that it exists */
+int g_suppress_equation_number = FALSE;
+int g_aux_file_missing = FALSE;    /* assume that it exists */
+int g_bbl_file_missing = FALSE;    /* assume that it exists */
 
-bool g_document_type = FORMAT_ARTICLE;
+int g_document_type = FORMAT_ARTICLE;
 int g_document_bibstyle = BIBSTYLE_STANDARD;
 
-bool g_fields_use_EQ = TRUE;
-bool g_fields_use_REF = TRUE;
-
 int g_safety_braces = 0;
-bool g_processing_equation = FALSE;
-bool g_RTF_warnings = FALSE;
+int g_processing_equation = FALSE;
+int g_RTF_warnings = FALSE;
 char *g_config_path = NULL;
 char *g_script_dir = NULL;
 char *g_tmp_dir = NULL;
 char *g_preamble = NULL;
-char g_field_separator = ',';
-bool g_escape_parens = FALSE;
+int g_escape_parens = FALSE;
+char *g_package_babel = NULL;
 
-bool g_equation_display_rtf = TRUE;
-bool g_equation_inline_rtf = TRUE;
-bool g_equation_inline_bitmap = FALSE;
-bool g_equation_display_bitmap = FALSE;
-bool g_equation_comment = FALSE;
-bool g_equation_raw_latex = FALSE;
-bool g_tableofcontents = FALSE;
+int g_equation_display_rtf = TRUE;
+int g_equation_inline_rtf = TRUE;
+int g_equation_inline_bitmap = FALSE;
+int g_equation_display_bitmap = FALSE;
+int g_equation_comment = FALSE;
+int g_equation_raw_latex = FALSE;
+int g_equation_mtef = FALSE;
+int g_tableofcontents = FALSE;
 
-bool g_tabular_display_rtf = TRUE;
-bool g_tabular_display_bitmap = FALSE;
+int g_tabular_display_rtf = TRUE;
+int g_tabular_display_bitmap = FALSE;
+int g_tab_counter = 0;
+int g_processing_table = FALSE;
+int g_processing_tabbing = FALSE;
+int g_processing_tabular = FALSE;
 
-double g_png_equation_scale = 1.22;
-double g_png_figure_scale = 1.35;
-bool g_latex_figures = FALSE;
-bool g_endfloat_figures = FALSE;
-bool g_endfloat_tables = FALSE;
-bool g_endfloat_markers = TRUE;
+double g_png_equation_scale = 1.00;
+double g_png_figure_scale = 1.00;
+int g_latex_figures = FALSE;
+int g_endfloat_figures = FALSE;
+int g_endfloat_tables = FALSE;
+int g_endfloat_markers = TRUE;
 int  g_graphics_package = GRAPHICS_NONE;
 
 int indent = 0;
 char alignment = JUSTIFIED;     /* default for justified: */
 
 int RecursionLevel = 0;
-bool twocolumn = FALSE;
-bool titlepage = FALSE;
+int twocolumn = FALSE;
+int titlepage = FALSE;
 
 static void OpenRtfFile(char *filename, FILE ** f);
 static void CloseRtf(FILE ** f);
@@ -162,8 +166,8 @@ int main(int argc, char **argv)
 
     InitializeStack();
     InitializeLatexLengths();
-	InitializeBibliography();
-	
+    InitializeBibliography();
+    
     while ((c = my_getopt(argc, argv, "lhpuvFSVWZ:o:a:b:d:f:i:s:u:C:D:M:P:T:t:")) != EOF) {
         switch (c) {
             case 'a':
@@ -181,14 +185,14 @@ int main(int argc, char **argv)
                 break;
             case 'f':
                 sscanf(optarg, "%d", &x);
-                g_fields_use_EQ = (x & 1) ? TRUE : FALSE;
-                g_fields_use_REF = (x & 2) ? TRUE : FALSE;
+                set_fields_use_EQ(x & 1);
+                set_fields_use_REF(x & 2);
                 break;
             case 'i':
-                setPackageBabel(optarg);
+                g_package_babel = strdup(optarg);
                 break;
             case 'l':
-                setPackageBabel("latin1");
+                g_package_babel = strdup("latin");
                 break;
             case 'o':
                 g_rtf_name = strdup(optarg);
@@ -203,7 +207,8 @@ int main(int argc, char **argv)
                 setPackageInputenc(optarg);
                 break;
             case 'D':
-                sscanf(optarg, "%d", &g_dots_per_inch);
+                sscanf(optarg, "%d", &x);
+                g_dots_per_inch = (uint16_t) x;
                 if (g_dots_per_inch < 25 || g_dots_per_inch > 600)
                     diagnostics(WARNING, "Dots per inch must be between 25 and 600 dpi\n");
                 break;
@@ -219,6 +224,7 @@ int main(int argc, char **argv)
                 g_equation_inline_bitmap = (x &  8) ? TRUE : FALSE;
                 g_equation_comment       = (x & 16) ? TRUE : FALSE;
                 g_equation_raw_latex     = (x & 32) ? TRUE : FALSE;
+                g_equation_mtef          = (x & 64) ? TRUE : FALSE;
                 diagnostics(3, "Math option g_equation_display_rtf    = %d", g_equation_display_rtf);
                 diagnostics(3, "Math option g_equation_inline_rtf     = %d", g_equation_inline_rtf);
                 diagnostics(3, "Math option g_equation_display_bitmap = %d", g_equation_display_bitmap);
@@ -309,10 +315,10 @@ int main(int argc, char **argv)
         diagnostics(ERROR, " Type \"latex2rtf -h\" for help");
     }
 
-/* Parse filename.	Extract directory if possible.	Beware of stdin cases */
+/* Parse filename.  Extract directory if possible.  Beware of stdin cases */
 
     if (argc == 1 && strcmp(*argv, "-") != 0) { /* filename exists and != "-" */
-        char *s, *t;
+        char *s, *t, *ext;
 
         basename = strdup(*argv);   /* parse filename */
         s = strrchr(basename, PATHSEP);
@@ -325,22 +331,16 @@ int main(int argc, char **argv)
             *(s + 1) = '\0';    /* g_home_dir = /tmp/ */
         }
 
-        t = strstr(basename, ".ltx");   /* remove .ltx if present */
-        if (t != NULL) {
-            *t = '\0';
-            g_tex_name = strdup_together(basename, ".ltx");
-
-        } else {
-
-            t = strstr(basename, ".tex");   /* remove .tex if present */
-            if (t != NULL)
-                *t = '\0';
-
+        /* remove .tex or .ltx if present */
+        ext = basename + strlen(basename) - 4;
+        if (strcmp(ext, ".tex") == 0 || strcmp(ext, ".ltx") == 0) {
+            g_tex_name = strdup(basename);
+            *ext = '\0';
+        } else 
             g_tex_name = strdup_together(basename, ".tex");
-        }
-
-        if (g_rtf_name == NULL)
-            g_rtf_name = strdup_together(basename, ".rtf");
+        
+        if (g_rtf_name == NULL) 
+            g_rtf_name = strdup_together3(g_home_dir,basename,".rtf");
     }
 
     if (g_aux_name == NULL && basename != NULL)
@@ -377,7 +377,7 @@ int main(int argc, char **argv)
     if (PushSource(g_tex_name, NULL) == 0) {
         OpenRtfFile(g_rtf_name, &fRtf);
 
-        InitializeDocumentFont(TexFontNumber("Roman"), 20, F_SHAPE_UPRIGHT, F_SERIES_MEDIUM);
+        InitializeDocumentFont(TexFontNumber("Roman"), 20, F_SHAPE_UPRIGHT, F_SERIES_MEDIUM, ENCODING_1252);
         PushTrackLineNumber(TRUE);
 
         ConvertWholeDocument();
@@ -385,9 +385,7 @@ int main(int argc, char **argv)
         CloseRtf(&fRtf);
         printf("\n");
 
-/*		debug malloc() 
-  		printf("Done!");
-  		while (1) {} do nothing */
+    if (0) debug_malloc();
 
         return 0;
     } else {
@@ -399,7 +397,7 @@ int main(int argc, char **argv)
 static void SetEndianness(void)
 
 /*
-purpose : Figure out endianness of machine.	 Needed for graphics support
+purpose : Figure out endianness of machine.  Needed for graphics support
 */
 {
     unsigned int endian_test = (unsigned int) 0xaabbccdd;
@@ -419,6 +417,8 @@ static void ConvertWholeDocument(void)
     PushEnvironment(PREAMBLE_MODE);
     setTexMode(MODE_VERTICAL);
     ConvertLatexPreamble();
+    setPackageBabel(g_package_babel);
+    
     WriteRtfHeader();
     ConvertString(t);
 
@@ -426,10 +426,10 @@ static void ConvertWholeDocument(void)
     preParse(&body, &sec_head, &label);
 
     diagnostics(2, "\\begin{document}");
-	diagnostics(5,"label for this section is'%s'", label);
-	diagnostics(5, "next section '%s'", sec_head);
-	show_string(2, body, "body ");
-	
+    diagnostics(5,"label for this section is'%s'", label);
+    diagnostics(5, "next section '%s'", sec_head);
+    show_string(2, body, "body ");
+    
     ConvertString(body);
     free(body);
     if (label)
@@ -444,10 +444,10 @@ static void ConvertWholeDocument(void)
             g_section_label = label;
         }
         
-		diagnostics(2, "processing '%s'", sec_head);	
-		diagnostics(5, "label is   '%s'", g_section_label);
-		diagnostics(5, "next  is   '%s'", sec_head2);	
-		show_string(2, body, "body ");
+        diagnostics(2, "processing '%s'", sec_head);    
+        diagnostics(5, "label is   '%s'", g_section_label);
+        diagnostics(5, "next  is   '%s'", sec_head2);   
+        show_string(2, body, "body ");
 
         ConvertString(sec_head);
         ConvertString(body);
@@ -457,38 +457,38 @@ static void ConvertWholeDocument(void)
     }
     
     if (g_endfloat_figures && g_fff_name) {
-    	g_endfloat_figures = FALSE;
+        g_endfloat_figures = FALSE;
         if (PushSource(g_fff_name, NULL) == 0) {
-        	CmdNewPage(NewPage);
-        	CmdListOf(LIST_OF_FIGURES);
-        	Convert();
+            CmdNewPage(NewPage);
+            CmdListOf(LIST_OF_FIGURES);
+            Convert();
         }
      }
     
     if (g_endfloat_tables && g_ttt_name) {
-    	g_endfloat_tables = FALSE;
+        g_endfloat_tables = FALSE;
         if (PushSource(g_ttt_name, NULL) == 0) {
-        	CmdNewPage(NewPage);
-        	CmdListOf(LIST_OF_TABLES);
-        	Convert();
+            CmdNewPage(NewPage);
+            CmdListOf(LIST_OF_TABLES);
+            Convert();
         }
      }
      
     if (strcmp(sec_head,"\\end{document}")==0) {
         diagnostics(2, "\\end{document}");
-    	ConvertString(sec_head);
-	}
+        ConvertString(sec_head);
+    }
 }
 
 static void print_version(void)
 {
     fprintf(stdout, "latex2rtf %s\n\n", Version);
-    fprintf(stdout, "Copyright (C) 2007 Free Software Foundation, Inc.\n");
+    fprintf(stdout, "Copyright (C) 2012 Free Software Foundation, Inc.\n");
     fprintf(stdout, "This is free software; see the source for copying conditions.  There is NO\n");
     fprintf(stdout, "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n\n");
     fprintf(stdout, "Written by Prahl, Lehner, Granzer, Dorner, Polzer, Trisko, Schlatterbeck.\n");
 
-/*		fprintf(stdout, "RTFPATH = '%s'\n", getenv("RTFPATH"));*/
+/*      fprintf(stdout, "RTFPATH = '%s'\n", getenv("RTFPATH"));*/
 }
 
 static void print_usage(void)
@@ -499,8 +499,8 @@ static void print_usage(void)
     fprintf(stdout, "Usage:  %s [options] input[.tex]\n\n", progname);
     fprintf(stdout, "Options:\n");
     fprintf(stdout, "  -a auxfile       use LaTeX auxfile rather than input.aux\n");
-    fprintf(stdout, "  -b bblfile       use BibTex bblfile rather than input.bbl)\n");
-    fprintf(stdout, "  -C codepage      latex encoding charset (latin1, cp850, raw, etc.)\n");
+    fprintf(stdout, "  -b bblfile       use BibTex bblfile rather than input.bbl\n");
+    fprintf(stdout, "  -C codepage      charset used by the latex document (latin1, cp850, raw, etc.)\n");
     fprintf(stdout, "  -d level         debugging output (level is 0-6)\n");
     fprintf(stdout, "  -f#              field handling\n");
     fprintf(stdout, "       -f0          do not use fields\n");
@@ -521,7 +521,7 @@ static void print_usage(void)
     fprintf(stdout, "       -M8          inline equations to bitmap\n");
     fprintf(stdout, "       -M12         inline and displayed equations to bitmaps\n");
     fprintf(stdout, "       -M16         insert Word comment field that the original equation text\n");
-    fprintf(stdout, "       -M32         insert the raw LaTeX equation delimited by <<: and :>>\n");
+    fprintf(stdout, "       -M32         insert the raw LaTeX equation delimited by $...$ and \\[...\\]\n");
     fprintf(stdout, "  -o outputfile    file for RTF output\n");
     fprintf(stdout, "  -p               option to avoid bug in Word for some equations\n");
     fprintf(stdout, "  -P path          paths to *.cfg & latex2png\n");
@@ -567,7 +567,7 @@ purpose: Writes the message to stderr depending on debugging level
     
     char buffer[512], *buff_ptr;
     va_list apf;
-    int i, iEnvCount;
+    int i;
 
     buff_ptr = buffer;
 
@@ -575,14 +575,14 @@ purpose: Writes the message to stderr depending on debugging level
 
     if (level <= g_verbosity_level) {
 
-        iEnvCount = CurrentEnvironmentCount();
+        CurrentEnvironmentCount();
 
-        if (!first) fprintf(stderr,"\n");
+        if (!first) fprintf(ERROUT,"\n");
         
-        fprintf(stderr, "%s:%-3d ",CurrentFileName(),CurrentLineNumber());
+        fprintf(ERROUT, "%s:%-3d ",CurrentFileName(),CurrentLineNumber());
         switch (level) {
             case 0:
-                fprintf(stderr, "Error! ");
+                fprintf(ERROUT, "Error! ");
                 break;
             case 1:
                 if (g_RTF_warnings) {
@@ -597,30 +597,30 @@ purpose: Writes the message to stderr depending on debugging level
                 break;
             case 5:
             case 6:
-                fprintf(stderr, " rec=%d ", RecursionLevel);
+                fprintf(ERROUT, " rec=%d ", RecursionLevel);
                 /*fall through */
             case 2:
             case 3:
             case 4:
                 for (i = 0; i < BraceLevel; i++)
-                    fprintf(stderr, "{");
+                    fprintf(ERROUT, "{");
                 for (i = 8; i > BraceLevel; i--)
-                    fprintf(stderr, " ");
+                    fprintf(ERROUT, " ");
 
                 for (i = 0; i < RecursionLevel; i++)
-                    fprintf(stderr, "  ");
+                    fprintf(ERROUT, "  ");
                 break;
             default:
                 break;
         }
-        vfprintf(stderr, format, apf);
-    	first = FALSE;
+        vfprintf(ERROUT, format, apf);
+        first = FALSE;
     }
     va_end(apf);
 
     if (level == 0) {
-        fprintf(stderr, "\n");
-        fflush(stderr);
+        fprintf(ERROUT, "\n");
+        fflush(ERROUT);
         if (fRtf) 
             fflush(fRtf);
             
@@ -644,6 +644,8 @@ static void InitializeLatexLengths(void)
     setLength("topmargin", 18 * 20);
     setLength("headsep", 25 * 20);
     setLength("textwidth", 345 * 20);
+    setLength("columnwidth", 345 * 20);
+    setLength("linewidth", 345 * 20);
     setLength("columnsep", 10 * 20);
     setLength("evensidemargin", 11 * 20);
 
@@ -653,6 +655,7 @@ static void InitializeLatexLengths(void)
     setLength("parskip", 0 * 20);
 
     setCounter("page", 0);
+    setCounter("part", 0);
     setCounter("chapter", 0);
     setCounter("section", 0);
     setCounter("subsection", 0);
@@ -703,20 +706,20 @@ purpose: reads the LaTeX preamble (to \begin{document} ) for the file
     char t[] = "\\begin|{|document|}";
     FILE *rtf_file;
 
-	/* Here we switch the file pointers ... it is important that nothing
-	   get printed to fRtf until the entire preamble has been processed.  
-	   This is really hard to track down, so the processed RTF get sent
-	   directly to stderr instead.
-	*/
-	rtf_file = fRtf;
-	fRtf = stderr;
-	
+    /* Here we switch the file pointers ... it is important that nothing
+       get printed to fRtf until the entire preamble has been processed.  
+       This is really hard to track down, so the processed RTF get sent
+       directly to stderr instead.
+    */
+    rtf_file = fRtf;
+    fRtf = ERROUT;
+    
     g_preamble = getSpacedTexUntil(t, 1);
 
     diagnostics(2, "Read LaTeX Preamble");
     diagnostics(5, "Entering ConvertString() from ConvertLatexPreamble");
 
-	show_string(5, g_preamble, "preamble");	
+    show_string(5, g_preamble, "preamble"); 
 
     ConvertString(g_preamble);
     diagnostics(5, "Exiting ConvertString() from ConvertLatexPreamble");
@@ -729,29 +732,21 @@ void OpenRtfFile(char *filename, FILE ** f)
 /****************************************************************************
 purpose: creates output file and writes RTF-header.
 params: filename - name of outputfile, possibly NULL for already open file
-	f - pointer to filepointer to store file ID
+    f - pointer to filepointer to store file ID
  ****************************************************************************/
 {
-    char *name;
-
     if (filename == NULL) {
         diagnostics(4, "Writing RTF to stdout");
         *f = stdout;
 
     } else {
 
-        if (g_home_dir)
-            name = strdup_together(g_home_dir, filename);
-        else
-            name = strdup(filename);
-
-        *f = fopen(name, "w");
+        *f = fopen(filename, "w");
 
         if (*f == NULL)
-            diagnostics(ERROR, "Error opening RTF file <%s>\n", name);
+            diagnostics(ERROR, "Error opening RTF file <%s>\n", filename);
 
-        diagnostics(2, "Opened RTF file <%s>", name);
-        free(name);
+        diagnostics(2, "Opened RTF file <%s>", filename);
     }
 }
 
@@ -772,8 +767,8 @@ globals: g_tex_name;
         diagnostics(WARNING, "\\section{}s.  For example ");
         diagnostics(WARNING, "   \\begin{small} ... \\section{A} ... \\section{B} ... \\end{small}");
         diagnostics(WARNING, "will definitely fail.");
-	}
-	
+    }
+    
     if (BraceLevel - 1 > g_safety_braces)
         diagnostics(WARNING, "Try translating with 'latex2rtf -Z%d %s'", BraceLevel - 1, g_tex_name);
 
@@ -787,7 +782,7 @@ globals: g_tex_name;
     }
     *f = NULL;
     diagnostics(4, "Closed RTF file");
-    fprintf(stderr,"\n");
+    fprintf(ERROUT,"\n");
 }
 
 void putRtfCharEscaped(char cThis)
@@ -797,9 +792,9 @@ purpose: output a single escaped character to the RTF file
          this is primarily useful for the verbatim-like enviroments
  ****************************************************************************/
 {
-	if (getTexMode() == MODE_VERTICAL)
-		changeTexMode(MODE_HORIZONTAL);
-	if (cThis == '\\')
+    if (getTexMode() == MODE_VERTICAL)
+        changeTexMode(MODE_HORIZONTAL);
+    if (cThis == '\\')
         fprintRTF("%s","\\\\");
     else if (cThis == '{')
         fprintRTF("%s", "\\{");
@@ -817,8 +812,8 @@ purpose: output a string with escaped characters to the RTF file
  ****************************************************************************/
 void putRtfStrEscaped(const char * string)
 {
-	char *s = (char *) string;
-	if (string == NULL) return;
+    char *s = (char *) string;
+    if (string == NULL) return;
     while (*s) putRtfCharEscaped(*s++);
 }
 
@@ -832,7 +827,7 @@ purpose: output a formatted string to the RTF file.  It is assumed that the
  ****************************************************************************/
 {
     char buffer[1024];
-    char *text;
+    unsigned char *text;
     char last='\0';
     
     va_list apf;
@@ -840,23 +835,23 @@ purpose: output a formatted string to the RTF file.  It is assumed that the
     va_start(apf, format);
     vsnprintf(buffer, 1024, format, apf);
     va_end(apf);
-    text = buffer;
+    text = (unsigned char *) buffer;
 
     while (*text) {
 
-		WriteEightBitChar(text[0], fRtf);
-	
-		if (*text == '{' && last != '\\')
-			PushFontSettings();
-	
-		if (*text == '}' && last != '\\')
-			PopFontSettings();
-	
-		if (*text == '\\' && last != '\\')
-			MonitorFontChanges(text);
-	
-		last= *text;
-		text++;
+        WriteEightBitChar(text[0], fRtf);
+    
+        if (*text == '{' && last != '\\')
+            PushFontSettings();
+    
+        if (*text == '}' && last != '\\')
+            PopFontSettings();
+    
+        if (*text == '\\' && last != '\\')
+            MonitorFontChanges(text);
+    
+        last= *text;
+        text++;
     }
 }
 
@@ -866,36 +861,65 @@ char *getTmpPath(void)
 purpose: return the directory to store temporary files
  ****************************************************************************/
 {
-#if defined(MSDOS) || defined(MACINTOSH) || defined(__MWERKS__)
-
-    return strdup("");
-
-#else
-
-    char *t, *u;
-    char pathsep_str[2] = { PATHSEP, 0 };   /* for os2 or w32 "unix" compiler */
+    size_t n;
+    char *t = NULL;
+    char *u = NULL;
+    char pathsep_str[2] = { PATHSEP, 0 };
 
+#if defined(MSDOS)
     /* first use any temporary directory specified as an option */
-    if (g_tmp_dir)
+    if (g_tmp_dir) {
         t = strdup(g_tmp_dir);
 
     /* next try the environment variable TMPDIR */
-    else if ((u = getenv("TMPDIR")) != NULL)
-        t = strdup(u);
+    } else {
+        u = getenv("TMPDIR");
+        if (u != NULL)
+            t = strdup(u);
+    }
+
+    /* finally just return ".\\" */
+    if (t==NULL)
+        t = strdup(".\\");
+
+    /* append a final '/' if missing */
+    
+    n=strlen(t)-1;
+    
+    if (n > 1 && t[n] != PATHSEP) {
+        u = strdup_together(t, pathsep_str);
+        free(t);
+        t = u;
+    }
+
+#else
+   /* first use any temporary directory specified as an option */
+    if (g_tmp_dir) {
+        t = strdup(g_tmp_dir);
+
+    /* next try the environment variable TMPDIR */
+    } else {
+        u = getenv("TMPDIR");
+        if (u != NULL)
+            t = strdup(u);
+    }
 
     /* finally just return "/tmp/" */
-    else
+    if (t==NULL)
         t = strdup("/tmp/");
 
     /* append a final '/' if missing */
-    if (*(t + strlen(t) - 1) != PATHSEP) {
+    
+    n=strlen(t)-1;
+    
+    if (n > 1 && t[n] != PATHSEP) {
         u = strdup_together(t, pathsep_str);
         free(t);
-        return u;
+        t = u;
     }
 
-    return t;
 #endif
+    return t;
 }
 
 char *my_strdup(const char *str)
@@ -907,14 +931,12 @@ purpose: duplicate string --- exists to ease porting
     char *s = NULL;
     unsigned long strsize;
 
-    strsize = strlen(str);
-    s = (char *) malloc(strsize + 1);
+    strsize = strlen(str) + 1;
+    s = (char *) malloc(strsize);
     *s = '\0';
     if (s == NULL)
         diagnostics(ERROR, "Cannot allocate memory to duplicate string");
-    strcpy(s, str);
-
-/*	diagnostics(5,"ptr %x",(unsigned long)s);*/
+    my_strlcpy(s, str, strsize);
     return s;
 }
 
@@ -938,11 +960,11 @@ purpose: opens "g_home_dir/path"  and
     p = fopen(name, mode);
 
     if (p == NULL) {
-    	if (strstr(path, ".tex") != NULL)
-        	p = (FILE *) open_cfg(path, FALSE);
-	} else
-    	diagnostics(2, "Opened '%s'", name);
-	
+        if (strstr(path, ".tex") != NULL)
+            p = (FILE *) open_cfg(path, FALSE);
+    } else
+        diagnostics(2, "Opened '%s'", name);
+    
     if (p == NULL) {
         diagnostics(WARNING, "Cannot open '%s'", name);
         fflush(NULL);
@@ -956,7 +978,9 @@ void debug_malloc(void)
 {
     char c;
 
-    diagnostics(WARNING, "Malloc Debugging --- press return to continue");
+    diagnostics(2, "Malloc Debugging --- press return to continue");
     fflush(NULL);
+    if (2 <= g_verbosity_level) {
     fscanf(stdin, "%c", &c);
+    c++;}
 }
