@@ -45,10 +45,11 @@ typedef struct InputStackType {
     char *string_start;
     FILE *file;
     char *file_name;
-    long file_line;
+    int file_line;
 } InputStackType;
 
 #define PARSER_SOURCE_MAX 100
+#define SCAN_BUFFER_SIZE   5000
 
 static InputStackType g_parser_stack[PARSER_SOURCE_MAX];
 
@@ -67,8 +68,6 @@ static int g_parser_backslashes;
 static int g_track_line_number_stack[TRACK_LINE_NUMBER_MAX];
 static int g_track_line_number = -1;
 
-static void parseBracket();
-
 void PushTrackLineNumber(int flag)
 
 /***************************************************************************
@@ -125,7 +124,7 @@ int CurrentFileDescriptor(void)
 {
     int fd=0;
     if (g_parser_file)
-    	fd = fileno(g_parser_file);
+        fd = fileno(g_parser_file);
     
     return fd;
 }
@@ -145,15 +144,15 @@ char *CurrentFileName(void)
 }
 
 /*
-	The following two routines allow parsing of multiple files and strings
+    The following two routines allow parsing of multiple files and strings
 */
 
 int PushSource(const char *filename, const char *string)
 
 /***************************************************************************
  purpose:     change the source used by getRawTexChar() to either file or string
- 			  --> pass NULL for unused argument (both NULL means use stdin)
- 			  --> PushSource duplicates string
+              --> pass NULL for unused argument (both NULL means use stdin)
+              --> PushSource duplicates string
 ****************************************************************************/
 {
     char s[50];
@@ -271,8 +270,10 @@ void PopSource(void)
     char s[50];
     int i;
 
-    if (g_parser_depth < 0)
-        diagnostics(ERROR, "More PopSource() calls than PushSource() ");
+    if (g_parser_depth < 0) {
+        diagnostics(1, "Hmmm.  More PopSource() calls than PushSource() calls");
+        return;
+    }
 
     if (0) {
         diagnostics(WARNING, "Before PopSource** line=%d, g_parser_depth=%d, g_parser_include_level=%d",
@@ -365,36 +366,36 @@ void CmdInclude(int code)
     } else {                    /* \input gnu */
         i = 0;
         while (cNext != '\0' && !isspace(cNext)) {
-        	if (i<99) name[i] = (char) cNext;
-        	i++;
-        	cNext = getTexChar();
+            if (i<99) name[i] = (char) cNext;
+            i++;
+            cNext = getTexChar();
         }
         
         if (i<99) 
-        	name[i] = '\0';
-		else {
-        	name[99] = '\0';
-        	diagnostics(WARNING, "\\input filename '%s' more than 100 chars, skipping",name);
-        	return;
+            name[i] = '\0';
+        else {
+            name[99] = '\0';
+            diagnostics(WARNING, "\\input filename '%s' more than 100 chars, skipping",name);
+            return;
         }
  
         basename = strdup(name);
     }
 
-	if (strstr(basename, "german.sty") != NULL) {
-    	GermanMode = TRUE;
-     	PushEnvironment(GERMAN_MODE);
-     	free(basename);
-     	return;
+    if (strstr(basename, "german.sty") != NULL) {
+        GermanMode = TRUE;
+        PushEnvironment(GERMAN_MODE);
+        free(basename);
+        return;
 
     } else if (strstr(basename, "french.sty") != NULL) {
         FrenchMode = TRUE;
         PushEnvironment(FRENCH_MODE);
-     	free(basename);
-     	return;
-	}
-	
-    if (basename && strstr(basename, ".tex") == NULL)         /* append .tex if missing */
+        free(basename);
+        return;
+    }
+    
+    if (basename && strstr(basename, ".tex") == NULL && strstr(basename, ".ltx") == NULL)         /* append .tex if missing */
         texname = strdup_together(basename, ".tex");
 
     if (texname && PushSource(texname, NULL) == 0)            /* Try the .tex name first*/
@@ -403,10 +404,10 @@ void CmdInclude(int code)
     else if (basename && PushSource(basename, NULL) == 0)     /* Try the basename second*/
         diagnostics(WARNING, "Including file <%s>", basename);
 
-	/* \include{file} always starts a new page */
-	if (code == 0)
-		PushSource(NULL, "\\pagebreak ");
-		
+    /* \include{file} always starts a new page */
+    if (code == 0)
+        PushSource(NULL, "\\pagebreak ");
+        
     if (basename) free(basename);
     if (texname)  free(texname);
 }
@@ -417,7 +418,7 @@ void CmdInclude(int code)
 
 int getParserDepth(void)
 {
-	return g_parser_depth;
+    return g_parser_depth;
 }
 
 
@@ -426,7 +427,7 @@ char getRawTexChar(void)
 /***************************************************************************
  purpose:     get the next character from the input stream with minimal
               filtering  (CRLF or CR or LF ->  \n) and '\t' -> ' '
-			  it also keeps track of the line number
+              it also keeps track of the line number
               should only be used by \verb and \verbatim and getTexChar()
 ****************************************************************************/
 {
@@ -458,20 +459,20 @@ char getRawTexChar(void)
     } else {
 
         if (g_parser_string && *g_parser_string) {
-			thechar = *g_parser_string;
+            thechar = *g_parser_string;
 
-			/* convert CR, CRLF, or LF to \n */			
-			if (thechar == CR) {   
-				g_parser_string++;
-				thechar = *g_parser_string;
-				if (thechar != LF)
-					g_parser_string--;
-				thechar = '\n';
-			} else if (thechar == LF)
-				thechar = '\n';
-			else if (thechar == '\t')
-            	thechar = ' ';
-            	
+            /* convert CR, CRLF, or LF to \n */         
+            if (thechar == CR) {   
+                g_parser_string++;
+                thechar = *g_parser_string;
+                if (thechar != LF)
+                    g_parser_string--;
+                thechar = '\n';
+            } else if (thechar == LF)
+                thechar = '\n';
+            else if (thechar == '\t')
+                thechar = ' ';
+                
             g_parser_currentChar = thechar;
             g_parser_string++;
         } 
@@ -489,14 +490,14 @@ char getRawTexChar(void)
     g_parser_penultimateChar = g_parser_lastChar;
     g_parser_lastChar = g_parser_currentChar;
     if (0) {
-		if (g_parser_currentChar=='\n')
-			diagnostics(5,"getRawTexChar = <\\n>");
-		else if (g_parser_currentChar=='\0')
-			diagnostics(5,"getRawTexChar = <\\0> depth=%d, files=%d", g_parser_depth, g_parser_include_level);
-		else
-			diagnostics(5,"getRawTexChar = <%2c>",g_parser_currentChar);
-	}
-	/* if (g_parser_currentChar=='\0') exit(0);*/
+        if (g_parser_currentChar=='\n')
+            diagnostics(5,"getRawTexChar = <\\n>");
+        else if (g_parser_currentChar=='\0')
+            diagnostics(5,"getRawTexChar = <\\0> depth=%d, files=%d", g_parser_depth, g_parser_include_level);
+        else
+            diagnostics(5,"getRawTexChar = <%2c>",g_parser_currentChar);
+    }
+    /* if (g_parser_currentChar=='\0') exit(0);*/
     return g_parser_currentChar;
 }
 
@@ -538,7 +539,7 @@ char getTexChar()
 /***************************************************************************
  purpose:     get the next character from the input stream
               This should be the usual place to access the LaTeX file
-			  It filters the input stream so that % is handled properly
+              It filters the input stream so that % is handled properly
 ****************************************************************************/
 {
     char cThis;
@@ -557,14 +558,14 @@ char getTexChar()
         g_parser_backslashes++;
     else
         g_parser_backslashes = 0;
-	if (0) {
-		if (cThis=='\n')
-			diagnostics(6,"getRawTexChar = <\\n> backslashes=%d line=%ld", g_parser_backslashes, g_parser_line);
-		else if (cThis=='\0')
-			diagnostics(6,"getRawTexChar = <\\0> backslashes=%d line=%ld", g_parser_backslashes, g_parser_line);
-		else
-			diagnostics(6,"getRawTexChar = <%2c> backslashes=%d line=%ld",cThis, g_parser_backslashes, g_parser_line);
-	}
+    if (0) {
+        if (cThis=='\n')
+            diagnostics(6,"getRawTexChar = <\\n> backslashes=%d line=%ld", g_parser_backslashes, g_parser_line);
+        else if (cThis=='\0')
+            diagnostics(6,"getRawTexChar = <\\0> backslashes=%d line=%ld", g_parser_backslashes, g_parser_line);
+        else
+            diagnostics(6,"getRawTexChar = <%2c> backslashes=%d line=%ld",cThis, g_parser_backslashes, g_parser_line);
+    }
     return cThis;
 }
 
@@ -589,9 +590,9 @@ char getNonBlank(void)
 {
     char c;
 
-	c = getTexChar();
+    c = getTexChar();
     while (c == ' ' || c == '\n') {
-    	c = getTexChar();
+        c = getTexChar();
     }
     return c;
 }
@@ -621,7 +622,7 @@ void skipSpaces(void)
     ungetTexChar(c);
 }
 
-void	skipWhiteSpace(void)
+void    skipWhiteSpace(void)
 /***************************************************************************
  Description: skip over spaces and linefeeds
 ****************************************************************************/
@@ -650,46 +651,58 @@ int getSameChar(char c)
     return count;
 }
 
-char *getDelimitedText(char left, char right, bool raw)
+char *getDelimitedText(char left, char right, int raw)
 
 /******************************************************************************
   purpose: general scanning routine that allocates and returns a string
-  		   that is between "left" and "right" that accounts for escaping by '\'
-  		   
-  		   Example for getDelimitedText('{','}',TRUE) 
-  		   
-  		   "{the \{ is shown {\it by} a\\}" ----> "the \{ is shown {\it by} a\\"
-  		    
-  		    Note the missing opening brace in the example above
+           that is between "left" and "right" that accounts for escaping by '\'
+           
+           Example for getDelimitedText('{','}',TRUE) 
+           
+           "the \{ is shown {\it by} a\\} blah blah" ----> "the \{ is shown {\it by} a\\"
+            
+           Note the missing opening brace in the example above
+           
+           It turns out that for getDelimitedText('[',']',TRUE)
+           
+           "the \] is shown {]} a\\] blah blah blah" ----> "the \] is shown {]} a\\"
+          
  ******************************************************************************/
 {
-    char buffer[5000];
+    char buffer[SCAN_BUFFER_SIZE];
     int size = -1;
     int lefts_needed = 1;
-    char marker = ' ';
-    char last_char = ' ';
-
-    while (lefts_needed && size < 4999) {
+    int brace_level = 0;
+    int last_char_was_backslash = FALSE;
 
+    while (lefts_needed && size < SCAN_BUFFER_SIZE-1) {
         size++;
-        last_char = marker;
         buffer[size] = (raw) ? getRawTexChar() : getTexChar();
-        marker = buffer[size];
 
-        if (buffer[size] != right || last_char == '\\') {   /* avoid \} */
-            if (buffer[size] == left && last_char != '\\')  /* avoid \{ */
-                lefts_needed++;
-            else {
-                if (buffer[size] == '\\' && last_char == '\\')  /* avoid \\} */
-                    marker = ' ';
+        if (last_char_was_backslash)  {            /* ignore \{ etc.           */
+            if (buffer[size] == '\\') {            /* two backslashes in a row */
+                last_char_was_backslash = FALSE;   /* next char is not special */
+                continue;
             }
-        } else
-            lefts_needed--;
+        }
+
+        else if (buffer[size] == right && brace_level == 0) 
+        	lefts_needed--;
+        
+        else if (buffer[size] == '{') 
+        	brace_level++;
+        
+        else if (buffer[size] == '}') 
+        	brace_level--;
+
+        last_char_was_backslash = (buffer[size] == '\\') ? TRUE : FALSE;
     }
 
     buffer[size] = '\0';        /* overwrite final delimeter */
-    if (size == 4999)
-        diagnostics(ERROR, "Misplaced '%c' (Not found within 5000 chars)");
+    if (size == SCAN_BUFFER_SIZE-1) {
+        diagnostics(WARNING, "Could not find closing '%c' in %d chars", right, SCAN_BUFFER_SIZE);
+        return strdup(" NOT FOUND ");
+    }
 
     return strdup(buffer);
 }
@@ -705,7 +718,7 @@ void parseBrace(void)
     free(s);
 }
 
-void parseBracket(void)
+static void parseBracket(void)
 
 /****************************************************************************
   Description: Skip text to balancing close bracket
@@ -733,7 +746,7 @@ void CmdIgnoreParameter(int code)
     int regParmCount = code % 10;
     char cThis;
 
-    diagnostics(5, "CmdIgnoreParameter [%d] {%d}", optParmCount, regParmCount);
+    diagnostics(4, "CmdIgnoreParameter [%d] {%d}", optParmCount, regParmCount);
 
     while (regParmCount) {
         cThis = getNonBlank();
@@ -788,7 +801,7 @@ char *getSimpleCommand(void)
     if (buffer[0] != '\\')
         return NULL;
 
-	buffer[1] = getTexChar();
+    buffer[1] = getTexChar();
 
     for (size = 2; size < 127; size++) {
         buffer[size] = getRawTexChar(); /* \t \r '%' all end command */
@@ -813,7 +826,7 @@ char *getBracketParam(void)
 
 /******************************************************************************
   purpose: return bracketed parameter
-  			
+            
   \item[1]   --->  "1"        \item[]   --->  ""        \item the  --->  NULL
        ^                           ^                         ^
   \item [1]  --->  "1"        \item []  --->  ""        \item  the --->  NULL
@@ -849,10 +862,10 @@ static char *getBraceParam0(int raw_flag)
            ^                                     ^
      \bar{text}    --->  "text"              \bar text     --->  "t"
          ^                                       ^
-	_\alpha        ---> "\alpha"             _{\alpha}     ---> "\alpha"
-	 ^                                        ^
-	_2             ---> "2"                  _{2}          ---> "2"
-	 ^                                        ^
+    _\alpha        ---> "\alpha"             _{\alpha}     ---> "\alpha"
+     ^                                        ^
+    _2             ---> "2"                  _{2}          ---> "2"
+     ^                                        ^
  **************************************************************************/
 {
     char s[2], *text;
@@ -882,14 +895,25 @@ static char *getBraceParam0(int raw_flag)
 
 char *getBraceParam(void)
 {
-	return getBraceParam0(FALSE);
+    return getBraceParam0(FALSE);
 }
 
 char *getBraceRawParam(void)
 {
-	return getBraceParam0(TRUE);
+    return getBraceParam0(TRUE);
 }
 
+void ignoreBraceParam(void) {
+    char *p = getBraceParam();
+    if (NULL != p) free(p);
+}
+
+void  ignoreBracketParam(void) {
+    char *p = getBracketParam();
+    if (NULL != p) free(p);
+}
+
+
 char *getLeftRightParam(void)
 
 /**************************************************************************
@@ -914,8 +938,8 @@ char *getLeftRightParam(void)
                     return strdup(text);
                 }
             }
-            strcat(text + i, command);
-            i += strlen(command);
+            my_strlcat(text + i, command, 5000);
+            i += (int) strlen(command);
             if (i > 4950)
                 diagnostics(ERROR, "Contents of \\left .. \\right too large.");
             if (strcmp(command, "\\left") == 0)
@@ -945,9 +969,9 @@ char *getTexUntil(char *target, int raw)
     char buffer[BUFFSIZE];
     int last_i = -1;
     int i = 0;                  /* size of string that has been read */
-    size_t j = 0;               /* number of found characters */
-    bool end_of_file_reached = FALSE;
-    size_t len = strlen(target);
+    int j = 0;               /* number of found characters */
+    int end_of_file_reached = FALSE;
+    int len = (int) strlen(target);
 
     PushTrackLineNumber(FALSE);
 
@@ -1017,20 +1041,17 @@ char *getSpacedTexUntil(char *target, int raw)
     char buffer[BUFFSIZE];
     char *s;
     int buffer_pos, target_pos, target_len, max_buffer_pos, start_pos;
-    bool end_of_file_reached = FALSE;
-	bool matched;
-	
+    
     PushTrackLineNumber(FALSE);
 
-    diagnostics(5, "getSpaceTexUntil target = <%s> raw_search = %d ", target, raw);
+    diagnostics(5, "getSpacedTexUntil target = <%s> raw_search = %d ", target, raw);
 
-	matched = FALSE;	
-	buffer_pos = 0;
-	target_pos = 0;
-	start_pos  = 0;
-	target_len = strlen(target);
-	max_buffer_pos = -1;
-	
+    buffer_pos = 0;
+    target_pos = 0;
+    start_pos  = 0;
+    target_len = (int) strlen(target);
+    max_buffer_pos = -1;
+    
     do {
         
         /* the next character might already be in the buffer */
@@ -1040,14 +1061,13 @@ char *getSpacedTexUntil(char *target, int raw)
         } 
 
         if (buffer[buffer_pos] == '\0') {
-            end_of_file_reached = TRUE;
             diagnostics(ERROR, "end of file reached before '%s' was found",target);
         }
 
         if (buffer[buffer_pos] == target[target_pos]) {
-        	if (target_pos == 0) 
-        		start_pos = buffer_pos;
-        	target_pos++;
+            if (target_pos == 0) 
+                start_pos = buffer_pos;
+            target_pos++;
         } 
         
         /* does not match next character in target ... */
@@ -1059,34 +1079,34 @@ char *getSpacedTexUntil(char *target, int raw)
         
         /* next character in target is '|' */ 
         } else if (buffer[buffer_pos] != ' ' && buffer[buffer_pos] != '\n') {
-        	
-			/* next char is non-blank ... either match or reset */
-			target_pos++;  /* move past wildcard */
-			if (buffer[buffer_pos] == target[target_pos]) {
-				target_pos++;
-			} else {
-				buffer_pos = start_pos;
-				target_pos = 0;
-			}
+            
+            /* next char is non-blank ... either match or reset */
+            target_pos++;  /* move past wildcard */
+            if (buffer[buffer_pos] == target[target_pos]) {
+                target_pos++;
+            } else {
+                buffer_pos = start_pos;
+                target_pos = 0;
+            }
         }
         
         if (0) {
-		if (buffer[buffer_pos] != '\n')
-			diagnostics(WARNING, "this char = <%c>, %d, %d, max=%d", buffer[buffer_pos], buffer_pos, target_pos, max_buffer_pos);
-		else
-			diagnostics(WARNING, "this char = <\\n>, %d, %d, max=%d", buffer[buffer_pos], buffer_pos, target_pos, max_buffer_pos);
-		}
-		
+        if (buffer[buffer_pos] != '\n')
+            diagnostics(WARNING, "this char = <%c>, %d, %d, max=%d", buffer[buffer_pos], buffer_pos, target_pos, max_buffer_pos);
+        else
+            diagnostics(WARNING, "this char = <\\n>, %d, %d, max=%d", buffer[buffer_pos], buffer_pos, target_pos, max_buffer_pos);
+        }
+        
         buffer_pos++;
 
 
-		if (buffer_pos == BUFFSIZE)
-			diagnostics(ERROR, "Could not find <%s> in %d characters \n\
-			Recompile with larger BUFFSIZE in getTexUntil() in parser.c", target, BUFFSIZE);
+        if (buffer_pos == BUFFSIZE)
+            diagnostics(ERROR, "Could not find <%s> in %d characters \n\
+            Recompile with larger BUFFSIZE in getTexUntil() in parser.c", target, BUFFSIZE);
 
     } while (target_pos < target_len);
 
-	/* terminate buffer */
+    /* terminate buffer */
     buffer[start_pos] = '\0';
 
     PopTrackLineNumber();
@@ -1114,17 +1134,17 @@ int getDimension(void)
 
 /* skip "to" */
     if (cThis == 't') {
-        cThis = getTexChar();
+        getTexChar();
         cThis = getTexChar();
     }
 
 /* skip "spread" */
     if (cThis == 's') {
-        cThis = getTexChar();
-        cThis = getTexChar();
-        cThis = getTexChar();
-        cThis = getTexChar();
-        cThis = getTexChar();
+        getTexChar();
+        getTexChar();
+        getTexChar();
+        getTexChar();
+        getTexChar();
         cThis = getTexChar();
     }
 
@@ -1155,20 +1175,20 @@ int getDimension(void)
         return 0;
     }
 
-/*	num *= 2;                    convert pts to twips */
+/*  num *= 2;                    convert pts to twips */
 
 /* obtain unit of measure */
     skipSpaces();
     buffer[0] = tolower((int) getTexChar());
 
-	if (buffer[0] == '\0')  /* no units specified ... assume points */
+    if (buffer[0] == '\0')  /* no units specified ... assume points */
         return (int) (num * 20);
-	
+    
 /* skip "true" */
     if (buffer[0] == 't') {
-        cThis = getTexChar();
-        cThis = getTexChar();
-        cThis = getTexChar();
+        getTexChar();
+        getTexChar();
+        getTexChar();
         skipSpaces();
         buffer[0] = tolower((int) getTexChar());
     }
@@ -1223,3 +1243,24 @@ int getDimension(void)
 
 }
 
+/***************************************************************************
+ purpose: return twips for \\, \\[1pt], \\*[1pt] 
+ ***************************************************************************/
+int getSlashSlashParam(void)
+{
+    char cThis, *vertical_space;
+    int height = 0;
+    
+    cThis = getTexChar();
+    if (cThis != '*')
+        ungetTexChar(cThis);
+
+    vertical_space = getBracketParam();
+    if (vertical_space) { 
+        height = getStringDimension(vertical_space);
+        free(vertical_space);
+    }
+    
+    return height;
+}
+
