@@ -23,10 +23,26 @@
  * Authors: 1995-1997 Ralf Schlatterbeck 1998-2000 Georg Lehner 2001-2002 Scott
  * Prahl
  */
+#if defined(NOSTDERR)
+#define ERROUT stdout
+#else
+#define ERROUT stderr
+#endif 
 
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
+
+#ifdef UNIX
+# include <sys/types.h>
+# include <sys/stat.h>
+# include <unistd.h>
+#else
+# include <stdio.h>
+#endif
+
+#include "cfg.h"
 #include "main.h"
 #include "utils.h"
 #include "parser.h"
@@ -34,17 +50,17 @@
 /******************************************************************************
  purpose:  returns true if n is odd
 ******************************************************************************/
-int odd(int n)
+int odd(long n)
 {
-    return (n & 1);
+    return (int) (n & 1);
 }
 
 /******************************************************************************
  purpose:  returns true if n is even
 ******************************************************************************/
-int even(int n)
+int even(long n)
 {
-    return (!(n & 1));
+    return (int) (!(n & 1));
 }
 
 /******************************************************************************
@@ -57,6 +73,19 @@ double my_rint(double nr)
   return (((c-nr) >= (nr-f)) ? f :c);
 }
 
+/******************************************************************************
+ purpose: this works with overlapping pointers ... the C standard says that
+   strcpy can do anything it likes for this case (which it happens to
+   do under Fedora 
+******************************************************************************/
+char *my_strcpy(char *dest, const char *src)
+{
+   char *save = dest;
+   while ( (*dest++ = *src++) );
+   return save;
+} 
+
+
 /******************************************************************************
  purpose:  count the number of occurences of the string t in the string s
 ******************************************************************************/
@@ -81,7 +110,8 @@ int strstr_count(const char *s, char *t)
 }
 
 /******************************************************************************
- purpose:  returns a new string with n characters from s (with '\0' at the end)
+ purpose:  returns a new string having n characters from src terminated with
+           '\0' at the end.  (so the length is n+1)
 ******************************************************************************/
 char *my_strndup(const char *src, size_t n)
 {
@@ -102,7 +132,8 @@ char *my_strndup(const char *src, size_t n)
 char *strdup_together(const char *s, const char *t)
 {
     char *both;
-
+    size_t siz;
+    
     if (s == NULL) {
         if (t == NULL)
             return NULL;
@@ -111,12 +142,16 @@ char *strdup_together(const char *s, const char *t)
     if (t == NULL)
         return strdup(s);
 
-    both = malloc(strlen(s) + strlen(t) + 1);
+    if (0) diagnostics(1, "'%s' + '%s'", s, t);
+    siz = strlen(s) + strlen(t) + 1;
+    both = (char *) malloc(siz);
+
     if (both == NULL)
         diagnostics(ERROR, "Could not allocate memory for both strings.");
 
-    strcpy(both, s);
-    strcat(both, t);
+    my_strlcpy(both, s, siz);
+    my_strlcat(both, t, siz);
+
     return both;
 }
 
@@ -126,10 +161,10 @@ char *strdup_together(const char *s, const char *t)
 char *strdup_together3(const char *s, const char *t, const char *u)
 {
     char *two, *three;
-	two = strdup_together(s,t);
-	three = strdup_together(two,u);
-	free(two);
-	return three;
+    two = strdup_together(s,t);
+    three = strdup_together(two,u);
+    free(two);
+    return three;
 }
 
 /******************************************************************************
@@ -138,10 +173,10 @@ char *strdup_together3(const char *s, const char *t, const char *u)
 char *strdup_together4(const char *s, const char *t, const char *u, const char *v)
 {
     char *four, *three;
-	three = strdup_together3(s,t,u);
-	four = strdup_together(three,v);
-	free(three);
-	return four;
+    three = strdup_together3(s,t,u);
+    four = strdup_together(three,v);
+    free(three);
+    return four;
 }
 
 /******************************************************************************
@@ -149,13 +184,13 @@ char *strdup_together4(const char *s, const char *t, const char *u, const char *
 ******************************************************************************/
 char *strdup_nocomments(const char *s)
 {
-    char *p, *dup;
+    char *p, *duplicate;
 
     if (s == NULL)
         return NULL;
 
-    dup = malloc(strlen(s) + 1);
-    p = dup;
+    duplicate = (char *) malloc(strlen(s) + 1);
+    p = duplicate;
 
     while (*s) {
         while (*s == '%') {     /* remove comment */
@@ -172,7 +207,7 @@ char *strdup_nocomments(const char *s)
     }
   done:
     *p = '\0';
-    return dup;
+    return duplicate;
 }
 
 /******************************************************************************
@@ -180,14 +215,14 @@ char *strdup_nocomments(const char *s)
 ******************************************************************************/
 char *strdup_noblanks(const char *s)
 {
-    char *p, *dup;
+    char *p, *duplicate;
 
     if (s == NULL)
         return NULL;
     while (*s == ' ' || *s == '\n')
         s++;                    /* skip to non blank */
-    dup = malloc(strlen(s) + 1);
-    p = dup;
+    duplicate = (char *) malloc(strlen(s) + 1);
+    p = duplicate;
     while (*s) {
         *p = *s;
         if (*p != ' ' && *p != '\n')
@@ -195,7 +230,7 @@ char *strdup_noblanks(const char *s)
         s++;
     }
     *p = '\0';
-    return dup;
+    return duplicate;
 }
 
 /*************************************************************************
@@ -203,10 +238,10 @@ purpose: duplicate text with only a..z A..Z 0..9 and _
 **************************************************************************/
 char *strdup_nobadchars(const char *text)
 {
-    char *dup, *s;
+    char *duplicate, *s;
 
-    dup = strdup_noblanks(text);
-    s = dup;
+    duplicate = strdup_noblanks(text);
+    s = duplicate;
 
     while (*s) {
         if (!('a' <= *s && *s <= 'z') && !('A' <= *s && *s <= 'Z') && !('0' <= *s && *s <= '9'))
@@ -214,7 +249,7 @@ char *strdup_nobadchars(const char *text)
         s++;
     }
 
-    return dup;
+    return duplicate;
 }
 
 /******************************************************************************
@@ -222,31 +257,31 @@ char *strdup_nobadchars(const char *text)
 ******************************************************************************/
 char *strdup_printable(const char *s)
 {
-    char *dup;
+    char *duplicate;
     int i;
     
     if (s == NULL) return NULL;
 
-    dup = malloc(2*strlen(s));
+    duplicate = (char *) malloc(2*strlen(s));
 
     i=0;
     while (*s) {
         if (*s=='\r') {
-            dup[i++]='\\';
-            dup[i++]='r';
+            duplicate[i++]='\\';
+            duplicate[i++]='r';
          } else if (*s=='\n') {
-            dup[i++]='\\';
-            dup[i++]='n';
+            duplicate[i++]='\\';
+            duplicate[i++]='n';
          } else if (*s=='\t') {
-            dup[i++]='\\';
-            dup[i++]='t';
+            duplicate[i++]='\\';
+            duplicate[i++]='t';
          } else 
-            dup[i++]=*s;
+            duplicate[i++]=*s;
          s++;
     }
-    dup[i]='\0';
+    duplicate[i]='\0';
         
-    return dup;
+    return duplicate;
 }
 
 /******************************************************************************
@@ -290,12 +325,12 @@ char *strdup_noendblanks(const char *s)
     if (*s == '\0')
         return strdup("");
 
-	/* find pointer to first non-space character in string */
+    /* find pointer to first non-space character in string */
     t = (char *) s;
     while (*t == ' ' || *t == '\n')
         t++;                    /* first non blank char */
 
-	/* find pointer to last non-space character in string */
+    /* find pointer to last non-space character in string */
     p = (char *) s + strlen(s) - 1;
     while (p >= t && (*p == ' ' || *p == '\n'))
         p--;                    /* last non blank char */
@@ -304,6 +339,42 @@ char *strdup_noendblanks(const char *s)
         return strdup("");
     return my_strndup(t, (size_t) (p - t + 1));
 }
+
+/******************************************************************************
+ purpose:  replaces all occurences of target in s with blanks
+******************************************************************************/
+void str_blank_out(char *s, const char *target)
+{
+    char *p;
+    int i, n;
+
+    if (s == NULL || target == NULL || *target == '\0')
+        return;
+    
+    n = strlen(target);
+    p = strstr(s,target);
+    while (p) {
+        for (i=0; i<n; i++) 
+            p[i] = ' ';
+        p = strstr(p+n,target);
+    }
+}
+
+/******************************************************************************
+ purpose:  deletes all occurences of target in s
+******************************************************************************/
+void str_delete(char *s, const char *target)
+{
+    char *p;
+    int n;
+    if (s == NULL || target == NULL || *target == '\0') return;   
+    
+    n = strlen(target);
+    p = s;
+    while ((p = strstr(p,target))) 
+        p = my_strcpy(p,p+n);
+}
+
 /******************************************************************************
   purpose: return a copy of tag from \label{tag} in the string text
  ******************************************************************************/
@@ -328,47 +399,75 @@ char *ExtractLabelTag(const char *text)
     return label;
 }
 
-/******************************************************************************
+/**************************************************************************
  purpose: provide functionality of getBraceParam() for strings
  
- 		if s contains "aaa {stuff}cdef", then  
- 			parameter = getStringBraceParam(&s)
- 			
- 		  gives
- 			parameter = "stuff"
- 			s="cdef"
+        if s contains "aaa {stuff}cdef", then  
+            parameter = getStringBraceParam(&s)
+            
+          gives
+            parameter = "stuff"
+            s="cdef"
+
+     \alpha\beta   --->  "\beta"             \bar \alpha   --->  "\alpha"
+           ^                                     ^
+     \bar{text}    --->  "text"              \bar text     --->  "t"
+         ^                                       ^
+    _\alpha        ---> "\alpha"             _{\alpha}     ---> "\alpha"
+     ^                                        ^
+    _2             ---> "2"                  _{2}          ---> "2"
+     ^                                        ^
  ******************************************************************************/
 char *getStringBraceParam(char **s)
 
 {
-	char *p_start, *p, *parameter, last;
-	int braces = 1;
-		
-	/* find start of parameter */
-	if (*s == NULL) return NULL;
-	p_start = strchr(*s,'{');
-	if (p_start==NULL) return NULL;
+    char *p_start, *p, *parameter, last;
+    int braces;
+    
+    if (*s == NULL) return strdup("");
+    
+    /* skip white space ... and one possible newline*/
+    while (**s == ' ') (*s)++;
+    if (**s == '\n') {
+        while (**s == ' ') (*s)++;
+    }
+    
+    p_start = *s;
 
-	/* scan to enclosing brace */
-	p_start++;
-	p=p_start;	
-	last = '\0';
-	while (*p != '\0' && braces > 0) {
-		if (*p == '{' && last != '\\')
-			braces++;
-		if (*p == '}' && last != '\\')
-			braces--;
-		last = *p;
-		p++;
-	}
-	
-	parameter = my_strndup(p_start, p-p_start-1);
-	*s = p;
+    /* return simple command like \alpha */
+    if (**s == '\\') {
+        do { (*s)++; } while (isalpha(**s));
+    diagnostics(1,"getstringbraceparam \\ before='%s'", *s);
+        return my_strndup(p_start,(*s)-p_start);
+    }
+    
+    /* no brace ... advance one and return next character */
+    if (**s != '{' ) {
+        (*s)++; 
+        return my_strndup(p_start,1);
+     }
+    
+    /* usual case, return contents between braces */
+    p_start++;
+    p=p_start;  
+    last = '\0';
+    braces = 1;
+    while (*p != '\0' && braces > 0) {
+        if (*p == '{' && last != '\\')
+            braces++;
+        if (*p == '}' && last != '\\')
+            braces--;
+        last = *p;
+        p++;
+    }
+    
+    parameter = my_strndup(p_start, p-p_start-1);
+    *s = p;
 
-	diagnostics(6,"Extract parameter=<%s> after=<%s>", parameter, *s); 
-	
-	return parameter;
-	
+    diagnostics(6,"Extract parameter=<%s> after=<%s>", parameter, *s); 
+    
+    return parameter;
+    
 }
 
 
@@ -395,10 +494,10 @@ char *ExtractAndRemoveTag(char *tag, char *text)
             break;
     }
 
-	contents = getStringBraceParam(&s);
-	if (contents == NULL) return NULL;
-	
-	/* erase "tag{contents}" */
+    contents = getStringBraceParam(&s);
+    if (contents == NULL) return NULL;
+    
+    /* erase "tag{contents}" */
     do
         *start++ = *s++;
     while (*s);               
@@ -423,90 +522,253 @@ char *ExtractAndRemoveTag(char *tag, char *text)
    
 char * keyvalue_pair(char *s, char **key, char **value)
 {
-	char *k, *v;	
-	*key = NULL;
-	*value = NULL;
-	if (s==NULL) return NULL;
-		
-	/* skip any blanks at start */
-	while (*s == ' ') s++;
+    char *k, *v;    
+    *key = NULL;
+    *value = NULL;
+    if (s==NULL) return NULL;
+        
+    /* skip any blanks at start */
+    while (*s == ' ') s++;
 
-	if (*s=='\0') return NULL;  /*possibly all blanks*/
-	
-	/* find the end of the key */
-	k = s;
-	while (*k != '=' && *k != ',' && *k != '\0') 
-		k++;
-		
-	/* allocate and copy string into the key */
-	*key = my_strndup(s, k-s);
-	
-	if (*k == '\0') return NULL;
-	
-	if (*k == ',') return k+1;
-	
-	/* '=' found, now parse value */
-	s = k+1;
-	
-	/* skip any blanks at start */
-	while (*s == ' ') s++;
-	
-	/* find the end of the value */
-	v = s;
-	while (*v != ',' && *v != '\0') {
-	    if (*v == '{')
-		while (*(++v) != '}')
-		    ;
-	    else
-		v++;
-	}
-	/* allocate and copy this into the value */
-	*value = my_strndup(s, v-s);
-	
-	if (*v == '\0') return NULL;
-	return v+1;
+    if (*s=='\0') return NULL;  /*possibly all blanks*/
+    
+    /* find the end of the key */
+    k = s;
+    while (*k != '=' && *k != ',' && *k != '\0') 
+        k++;
+        
+    /* allocate and copy string into the key */
+    *key = my_strndup(s, k-s);
+    
+    if (*k == '\0') return NULL;
+    
+    if (*k == ',') return k+1;
+    
+    /* '=' found, now parse value */
+    s = k+1;
+    
+    /* skip any blanks at start */
+    while (*s == ' ') s++;
+    
+    /* find the end of the value */
+    v = s;
+    while (*v != ',' && *v != '\0') {
+        if (*v == '{')
+        while (*(++v) != '}')
+            ;
+        else
+        v++;
+    }
+    /* allocate and copy this into the value */
+    *value = my_strndup(s, v-s);
+    
+    if (*v == '\0') return NULL;
+    return v+1;
 }
 
 int getStringDimension(char *s)
 {
-	int size = 0;
-	
-	if (s != NULL) {
-		PushSource(NULL, s);
-		size = getDimension();
-		PopSource();
-	}
-	
-	return size;
+    int size = 0;
+    
+    if (s != NULL) {
+        PushSource(NULL, s);
+        size = getDimension();
+        PopSource();
+    }
+    
+    diagnostics(5, "getStringDimension fore '%s' is %d twips", s, size);
+    return size;
 }
 
 void show_string(int level, const char *s, const char *label)
 {
-	int width=100;
-	long i;
-	char c;
-	long len;
-	
-	if (g_verbosity_level<level) return;
-		
-	if (s == NULL) {
-		diagnostics(WARNING, "\n%s: NULL",label);
-		return;
-	}
-		
-	len = strlen(s);
-	fprintf(stderr, "\n%s: ", label);
+    int width=100;
+    long i;
+    char c;
+    long len;
+    
+    if (g_verbosity_level<level) return;
+        
+    if (s == NULL) {
+        diagnostics(WARNING, "\n%s: NULL",label);
+        return;
+    }
+        
+    len = strlen(s);
+    fprintf(ERROUT, "\n%s: ", label);
 
-	for (i=0; i<len; i++) {
-	
-		if (i==width)
-			fprintf(stderr, "\n%-*d: ", (int) strlen(label), (int) strlen(s));
-		else if (i>1 && i % width == 0) 
-			fprintf(stderr, "\n%s: ",label);
-		c = s[i];
-		if (c == '\n') c = '=';
-		if (c == '\0') c = '*';
-		fprintf(stderr,"%c",c);
-	}
+    for (i=0; i<len; i++) {
+    
+        if (i==width)
+            fprintf(ERROUT, "\n%-*d: ", (int) strlen(label), (int) strlen(s));
+        else if (i>1 && i % width == 0) 
+            fprintf(ERROUT, "\n%s: ",label);
+        c = s[i];
+        if (c == '\n') c = '=';
+        if (c == '\0') c = '*';
+        fprintf(ERROUT,"%c",c);
+    }
 }
 
+/* these next two routines fall under the copyright banner below.  The
+names of the routines have had 'my_' prepended to avoid conflicting
+with existing versions in string.h  */
+
+/*
+ * Copyright (c) 1998 Todd C. Miller <Todd.Miller@courtesan.com>
+ *
+ * Permission to use, copy, modify, and distribute this software for any
+ * purpose with or without fee is hereby granted, provided that the above
+ * copyright notice and this permission notice appear in all copies.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
+ * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
+ * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
+ * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
+ * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ */
+
+/*
+ * Copy src to string dst of size siz.  At most siz-1 characters
+ * will be copied.  Always NUL terminates (unless siz == 0).
+ * Returns strlen(src); if retval >= siz, truncation occurred.
+ */
+size_t
+my_strlcpy(char *dst, const char *src, size_t siz)
+{
+        char *d = dst;
+        const char *s = src;
+        size_t n = siz;
+
+        /* Copy as many bytes as will fit */
+        if (n != 0) {
+                while (--n != 0) {
+                        if ((*d++ = *s++) == '\0')
+                                break;
+                }
+        }
+
+        /* Not enough room in dst, add NUL and traverse rest of src */
+        if (n == 0) {
+                if (siz != 0)
+                        *d = '\0';                /* NUL-terminate dst */
+                while (*s++)
+                        ;
+        }
+
+        return(s - src - 1);        /* count does not include NUL */
+}
+
+/*
+ * Appends src to string dst of size siz (unlike strncat, siz is the
+ * full size of dst, not space left).  At most siz-1 characters
+ * will be copied.  Always NUL terminates (unless siz <= strlen(dst)).
+ * Returns strlen(src) + MIN(siz, strlen(initial dst)).
+ * If retval >= siz, truncation occurred.
+ */
+size_t
+my_strlcat(char *dst, const char *src, size_t siz)
+{
+        char *d = dst;
+        const char *s = src;
+        size_t n = siz;
+        size_t dlen;
+
+        /* Find the end of dst and adjust bytes left but don't go past end */
+        while (n-- != 0 && *d != '\0')
+                d++;
+        dlen = d - dst;
+        n = siz - dlen;
+
+        if (n == 0)
+                return(dlen + strlen(s));
+        while (*s != '\0') {
+                if (n != 1) {
+                        *d++ = *s;
+                        n--;
+                }
+                s++;
+        }
+        *d = '\0';
+
+        return(dlen + (s - src));        /* count does not include NUL */
+}
+
+
+/*
+ * handy litte portable file existance check
+ */
+int file_exists(char *fname)
+{
+    int result = FALSE;
+#ifdef UNIX
+    struct stat fStat;
+    result = (stat(fname,&fStat) == 0);
+#else
+    FILE *f = fopen(fname,"rb");
+    if (NULL != f) {
+        result = TRUE;
+        fclose(f);
+    }
+#endif
+    diagnostics(5,"file_exists(%s) returns %d",fname,result);
+    return result;
+}
+
+#define CR (char) 0x0d
+#define LF (char) 0x0a
+
+int my_fgetc(FILE *f)
+{
+    int c;
+    
+    c = fgetc(f);
+    if (feof(f)) return '\0';
+        
+    if (c==CR) {
+        c = fgetc(f);
+        if (c == LF) return '\n';
+        ungetc(c,f);
+        return '\n';
+    } 
+    
+    if (c == LF ) return '\n';
+    if (c =='\t') return ' ';
+    
+    return c;
+}
+
+/* fgets function that honors '\' at end of line 
+
+      \harvardcite{aharanov1995}{Aharanov, Whitehead, Kelemen \harvardand \
+      Spiegelman}{Aharanov et~al.}{1995}
+      
+   returns number of characters read
+*/
+
+char * my_fgets(char *buffer, int maxBuffer, FILE *f) 
+{
+    int i;
+    int cLast = '\0';
+    
+    if (f == NULL || feof(f)) return NULL;
+    
+    for (i=0; i<maxBuffer; i++) {
+        buffer[i] = my_fgetc(f);
+    
+        if (buffer[i] == '\0') return buffer;
+        
+        if (buffer[i] == '\n') {
+            if (cLast == '\\')
+                buffer[i] = ' ';  /* replace backslash-newline with backslash-space */
+            else 
+                break;
+        } 
+        cLast = buffer[i];
+    }
+        
+    buffer[i] = '\0';
+    return buffer;
+}
